/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.7
  * @date     2022-02-11
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32f403a_407_clock.h"
#include "FreeRTOS.h"
#include "task.h"
#include "usart.h"

/* ʼȼ */
#define START_TASK_PRIO		1
/* ʼջС */
#define START_STK_SIZE 		128  
/* ʼ */
TaskHandle_t StartTask_Handler;
/* ʼں */
void start_task(void *pvParameters);

/* LED0ȼ */
#define LED0_TASK_PRIO		4
/* LED0ջС */
#define LED0_STK_SIZE 		128  
/* LED0 */
TaskHandle_t LED0Task_Handler;
/* LED0ں */
void led0_task(void *pvParameters);

/* LED1ȼ */
#define LED1_TASK_PRIO		3
/* LED1ջС */
#define LED1_STK_SIZE 		128  
/* LED1 */
TaskHandle_t LED1Task_Handler;
/* LED1ں */
void led1_task(void *pvParameters);

/* ȼ */
#define FLOAT_TASK_PRIO		2
/* ջС */
#define FLOAT_STK_SIZE 		256
/*  */
TaskHandle_t FLOATTask_Handler;
/* ں */
void float_task(void *pvParameters);

/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);
  
  system_clock_config();
  
  /* init usart1 */
  USART_Init(115200);
  
  at32_led_init(LED2);
  at32_led_init(LED3);
  at32_led_init(LED4);
  
  /* ʼ */
  xTaskCreate((TaskFunction_t )start_task,
              (const char*    )"start_task",
              (uint16_t       )START_STK_SIZE,
              (void*          )NULL,
              (UBaseType_t    )START_TASK_PRIO,
              (TaskHandle_t*  )&StartTask_Handler);  
  /*  */              
  vTaskStartScheduler();
}

/* ʼ */
void start_task(void *pvParameters)
{
  /* ٽ */
  taskENTER_CRITICAL();
  /* LED0 */
  xTaskCreate((TaskFunction_t )led0_task,     	
              (const char*    )"led0_task",   	
              (uint16_t       )LED0_STK_SIZE, 
              (void*          )NULL,				
              (UBaseType_t    )LED0_TASK_PRIO,	
              (TaskHandle_t*  )&LED0Task_Handler);   
  /* LED1 */
  xTaskCreate((TaskFunction_t )led1_task,     
              (const char*    )"led1_task",   
              (uint16_t       )LED1_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )LED1_TASK_PRIO,
              (TaskHandle_t*  )&LED1Task_Handler);        
  /*  */
  xTaskCreate((TaskFunction_t )float_task,     
              (const char*    )"float_task",   
              (uint16_t       )FLOAT_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )FLOAT_TASK_PRIO,
              (TaskHandle_t*  )&FLOATTask_Handler);  
  /* ɾʼ */
  vTaskDelete(StartTask_Handler);
  /* ˳ٽ */
  taskEXIT_CRITICAL();
}

/* LED0 */
void led0_task(void *pvParameters)
{
    while(1)
    {
      at32_led_toggle(LED3);
      printf("LED3 Toggle\r\n");
      vTaskDelay(1000);
    }
}

/* LED1ʼ */
void led1_task(void *pvParameters)
{
    while(1)
    {
      at32_led_toggle(LED2);
      printf("LED2 Toggle\r\n");
      vTaskDelay(500);
    }
}

/*  */
void float_task(void *pvParameters)
{
	static float float_num=0.00;
	while(1)
	{
		float_num+=0.01f;
		printf("float_num = %.4f\r\n",float_num);
    vTaskDelay(1000);
	}
}

