/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.7
  * @date     2022-02-11
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32f403a_407_clock.h"
#include "FreeRTOS.h"
#include "task.h"
#include "timer.h"
#include "usart.h"
#include "key.h"

/* ʼȼ */
#define START_TASK_PRIO		1
/* ʼջС */
#define START_STK_SIZE 		128  
/* ʼ */
TaskHandle_t StartTask_Handler;
/* ʼں */
void start_task(void *pvParameters);

/* LED0ȼ */
#define LED0_TASK_PRIO		4
/* LED0ջС */
#define LED0_STK_SIZE 		128  
/* LED0 */
TaskHandle_t LED0Task_Handler;
/* LED0ں */
void led0_task(void *pvParameters);

/* LED1ȼ */
#define LED1_TASK_PRIO		3
/* LED1ջС */
#define LED1_STK_SIZE 		128  
/* LED1 */
TaskHandle_t LED1Task_Handler;
/* LED1ں */
void led1_task(void *pvParameters);


/* ȼ */
#define Debug_TASK_PRIO		2
/* ջС	*/
#define Debug_STK_SIZE 		512
/*  */
TaskHandle_t DebugTask_Handler;
/* ں */
void debug_task(void *pvParameters);


/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);
  
  system_clock_config();
  
  at32_led_init(LED2);
  at32_led_init(LED3);
  at32_led_init(LED4);
  
  
  /* ʼںͰ */
  USART_Init(115200);
  KEY_Init();
  
  TIMER_Init();
  /* ʼ */
  xTaskCreate((TaskFunction_t )start_task,
              (const char*    )"start_task",
              (uint16_t       )START_STK_SIZE,
              (void*          )NULL,
              (UBaseType_t    )START_TASK_PRIO,
              (TaskHandle_t*  )&StartTask_Handler);  
  /*  */              
  vTaskStartScheduler();
}

/* ʼ */
void start_task(void *pvParameters)
{
  /* ٽ */
  taskENTER_CRITICAL();
  /* LED0 */
  xTaskCreate((TaskFunction_t )led0_task,     	
              (const char*    )"led0_task",   	
              (uint16_t       )LED0_STK_SIZE, 
              (void*          )NULL,				
              (UBaseType_t    )LED0_TASK_PRIO,	
              (TaskHandle_t*  )&LED0Task_Handler);   
  /* LED1 */
  xTaskCreate((TaskFunction_t )led1_task,     
              (const char*    )"led1_task",   
              (uint16_t       )LED1_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )LED1_TASK_PRIO,
              (TaskHandle_t*  )&LED1Task_Handler);               
  /* */
  xTaskCreate((TaskFunction_t )debug_task,     
              (const char*    )"Debug_task",   
              (uint16_t       )Debug_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Debug_TASK_PRIO,
              (TaskHandle_t*  )&DebugTask_Handler);                
  /* ɾʼ */
  vTaskDelete(StartTask_Handler);
  /* ˳ٽ */
  taskEXIT_CRITICAL();
}

/* LED0 */
void led0_task(void *pvParameters)
{
  while(1)
  {
    /* ٽ */
    taskENTER_CRITICAL();
    at32_led_toggle(LED3);
    printf("ٽ\r\n");
    /* ʹTMR5Ӳжģͨʱ,ʹvTaskDelay() */
    while(tmr_flag_get(TMR5,TMR_OVF_FLAG)==RESET);
    tmr_flag_clear(TMR5,TMR_OVF_FLAG);
    printf("˳ٽ\r\n");   
    /* ˳ٽ */
    taskEXIT_CRITICAL();
    vTaskDelay(1000);
  }
}

/* LED1 */
void led1_task(void *pvParameters)
{
  while(1)
  {
    at32_led_toggle(LED2);
    vTaskDelay(500);
  }
}

/*  */
void debug_task(void *pvParameters)
{
  u8 buff[500];
  while(1)
  {
    /* °ӡһϢ */
    if(key_press() == USER_BUTTON)
    {
      printf("/*-----------------------------------------------------------------------------*/\r\n");
      printf("Task                 Status          priority      Remaining_Stack     Num\r\n");
      vTaskList((char *)&buff);
      printf("%s\r\n",buff);
      printf("/*----------------------------------------------------------------------------*/\r\n");
      printf("Task        Runing_Num       Usage_Rate\r\n");
      vTaskGetRunTimeStats((char *)&buff);
      printf("%s\r\n",buff);
    }
    vTaskDelay(10);
  }
}
