/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.7
  * @date     2022-02-11
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32f403a_407_clock.h"
#include "FreeRTOS.h"
#include "task.h"
#include "timer.h"
#include "usart.h"
#include "key.h"

/* ʼȼ*/
#define START_TASK_PRIO		1
/* ʼջС */
#define START_STK_SIZE 		128  
/* ʼ */
TaskHandle_t StartTask_Handler;
/* ʼں */
void start_task(void *pvParameters);

/* 1ȼ */
#define TASK1_PRIO		5
/* 1ջС	*/
#define TASK1_STK_SIZE 		128
/* 1 */
TaskHandle_t TASK1_Handler;
/* 1ں */
void task1(void *pvParameters);

/* 2ȼ */
#define TASK2_PRIO		5
/* 2ջС	*/
#define TASK2_STK_SIZE 		128
/* 2 */
TaskHandle_t TASK2_Handler;
/* 2ں */
void task2(void *pvParameters);

/* 3ȼ */
#define TASK3_PRIO		5
/* 3ջС	*/
#define TASK3_STK_SIZE 		128
/* 3 */
TaskHandle_t TASK3_Handler;
/* 3ں */
void task3(void *pvParameters);


/* ȼ */
#define Debug_TASK_PRIO		2
/* ջС	*/
#define Debug_STK_SIZE 		512
/*  */
TaskHandle_t DebugTask_Handler;
/* ں */
void debug_task(void *pvParameters);


/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);
  
  system_clock_config();
  
  at32_led_init(LED2);
  at32_led_init(LED3);
  at32_led_init(LED4);
  
  
  /* ʼںͰ */
  USART_Init(115200);
  KEY_Init();
  
  TIMER_Init();
  /* ʼ */
  xTaskCreate((TaskFunction_t )start_task,
              (const char*    )"start_task",
              (uint16_t       )START_STK_SIZE,
              (void*          )NULL,
              (UBaseType_t    )START_TASK_PRIO,
              (TaskHandle_t*  )&StartTask_Handler);  
  /*  */              
  vTaskStartScheduler();
}

/* ʼ */
void start_task(void *pvParameters)
{
  /* ٽ */
  taskENTER_CRITICAL(); 
  /* task1 */
  xTaskCreate((TaskFunction_t )task1,     
              (const char*    )"task1",   
              (uint16_t       )TASK1_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )TASK1_PRIO,
              (TaskHandle_t*  )&TASK1_Handler);  
  /* task2 */
  xTaskCreate((TaskFunction_t )task2,     
              (const char*    )"task2",   
              (uint16_t       )TASK2_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )TASK2_PRIO,
              (TaskHandle_t*  )&TASK2_Handler);   
  /* task3 */
  xTaskCreate((TaskFunction_t )task3,     
              (const char*    )"task3",   
              (uint16_t       )TASK3_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )TASK3_PRIO,
              (TaskHandle_t*  )&TASK3_Handler);               
  /*  */
  xTaskCreate((TaskFunction_t )debug_task,     
              (const char*    )"Debug_task",   
              (uint16_t       )Debug_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Debug_TASK_PRIO,
              (TaskHandle_t*  )&DebugTask_Handler);
  /* ɾʼ */            
  vTaskDelete(StartTask_Handler); 
  /* ˳ٽ */            
  taskEXIT_CRITICAL();            
}
/* task1 */
void task1(void *pvParameters)
{
  while(1)
  {
    printf("task1 running\r\n");
  }
}
/* task2 */
void task2(void *pvParameters)
{
  while(1)
  {
    printf("task2 running\r\n");
  }
}
/* task3 */
void task3(void *pvParameters)
{
  while(1)
  {
    printf("task3 running\r\n");
  }
}


/*  */
void debug_task(void *pvParameters)
{
  u8 buff[500];
  while(1)
  {
    /* °ӡһεϢ */
    if(key_press() == USER_BUTTON)
    {
      printf("/*-----------------------------------------------------------------------------*/\r\n");
      printf("Task                 Status          priority      Remaining_Stack     Num\r\n");
      vTaskList((char *)&buff);
      printf("%s\r\n",buff);
      printf("/*----------------------------------------------------------------------------*/\r\n");
      printf("Task        Runing_Num       Usage_Rate\r\n");
      vTaskGetRunTimeStats((char *)&buff);
      printf("%s\r\n",buff);
    }
    vTaskDelay(10);
  }
}

