/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.7
  * @date     2022-02-11
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32f403a_407_clock.h"
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "timer.h"
#include "usart.h"
#include "key.h"

/* ʼȼ*/
#define START_TASK_PRIO		1
/* ʼջС */
#define START_STK_SIZE 		128  
/* ʼ */
TaskHandle_t StartTask_Handler;
/* ʼں */
void start_task(void *pvParameters);

/* ȼȼ */
#define High_Priority_TASK_PRIO		5
/* ȼջС */
#define High_Priority_STK_SIZE 		256  
/* ȼ */
TaskHandle_t High_Priority_Task_Handler;
/* ȼں */
void High_Priority_task(void *pvParameters);

/* ȼȼ */
#define Middle_Priority_TASK_PRIO		4
/* ȼջС */
#define Middle_Priority_STK_SIZE 		256  
/* ȼ */
TaskHandle_t Middle_Priority_Task_Handler;
/* ȼں */
void Middle_Priority_task(void *pvParameters);

/* ȼȼ */
#define Low_Priority_TASK_PRIO		3
/* ȼջС */
#define Low_Priority_STK_SIZE 		256  
/* ȼ */
TaskHandle_t Low_Priority_Task_Handler;
/* ȼں */
void Low_Priority_task(void *pvParameters);

/* ȼ */
#define Debug_TASK_PRIO		3
/* ջС	*/
#define Debug_STK_SIZE 		512
/*  */
TaskHandle_t DebugTask_Handler;
/* ں */
void debug_task(void *pvParameters);

/* ź */
SemaphoreHandle_t AT_xSemaphore;


/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);
  
  system_clock_config();
  
  at32_led_init(LED2);
  at32_led_init(LED3);
  at32_led_init(LED4);
  
  /* ʼںͰ */
  USART_Init(115200);
  KEY_Init();
  
  /* ʼ */
  xTaskCreate((TaskFunction_t )start_task,
              (const char*    )"start_task",
              (uint16_t       )START_STK_SIZE,
              (void*          )NULL,
              (UBaseType_t    )START_TASK_PRIO,
              (TaskHandle_t*  )&StartTask_Handler);  
  /*  */              
  vTaskStartScheduler();
}
/* ʼ */
void start_task(void *pvParameters)
{
  /* ٽ */
  taskENTER_CRITICAL(); 
  
  /* ź */
  AT_xSemaphore = xSemaphoreCreateMutex();
  
  if( AT_xSemaphore == NULL )
  {
    /* źʧ */
    while(1);
  }

  /* ڴź֮ٳʼʱ */
  TIMER_Init();
 
  /* ȼ */
  xTaskCreate((TaskFunction_t )High_Priority_task,     
              (const char*    )"High_task",   
              (uint16_t       )High_Priority_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )High_Priority_TASK_PRIO,
              (TaskHandle_t*  )&High_Priority_Task_Handler);    
  /* ȼ */
  xTaskCreate((TaskFunction_t )Middle_Priority_task,     
              (const char*    )"Middle_task",   
              (uint16_t       )Middle_Priority_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Middle_Priority_TASK_PRIO,
              (TaskHandle_t*  )&Middle_Priority_Task_Handler);     
  /* ȼ */
  xTaskCreate((TaskFunction_t )Low_Priority_task,     
              (const char*    )"Low_task",   
              (uint16_t       )Low_Priority_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Low_Priority_TASK_PRIO,
              (TaskHandle_t*  )&Low_Priority_Task_Handler);               
  /*  */
  xTaskCreate((TaskFunction_t )debug_task,     
              (const char*    )"Debug_task",   
              (uint16_t       )Debug_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Debug_TASK_PRIO,
              (TaskHandle_t*  )&DebugTask_Handler);
  /* ɾʼ */            
  vTaskDelete(StartTask_Handler); 
  /* ˳ٽ */            
  taskEXIT_CRITICAL();            
}

/* ȼ */
void High_Priority_task(void *pvParameters)
{
 
  while(1)
  {
    /* ʱ,øȼûź */  
    vTaskDelay(10);
    
    if(xSemaphoreTake( AT_xSemaphore, portMAX_DELAY ) == pdTRUE)
    {
      printf("High priority task received a mutex semaphore.\r\n");
    }
   
  }
}

void Middle_Priority_task(void *pvParameters)
{
  while(1)
  {
    printf("Middle priority task is running.\r\n");
    printf("Change Middle priority task to Block state.\r\n");    
    vTaskSuspend(Middle_Priority_Task_Handler); 
    vTaskDelay(10);
  }
  
}

void Low_Priority_task(void *pvParameters)
{
  while(1)
  {
    /* ʹȼûź */
    if(xSemaphoreTake( AT_xSemaphore, portMAX_DELAY ) == pdTRUE)
    {
      printf("Low priority task received a mutex semaphore.\r\n");
      printf("Change Middle priority task to Ready state.\r\n");
      /* ʹеȼ̬ */
      vTaskResume(Middle_Priority_Task_Handler);
      /* ʹöʱжϱ־ģͨʱ */
      while(tmr_flag_get(TMR3,TMR_OVF_FLAG)==RESET);
      tmr_flag_clear(TMR3,TMR_OVF_FLAG);
      
      printf("Low priority task release a mutex semaphore.\r\n");
      xSemaphoreGive( AT_xSemaphore );
      
    }
  }
   
}
/*  */
void debug_task(void *pvParameters)
{
  u8 buff[500];
  while(1)
  {
    /* °ӡһεϢ */
    if(key_press() == USER_BUTTON)
    {
      printf("/*-----------------------------------------------------------------------------*/\r\n");
      printf("Task                 Status          priority      Remaining_Stack     Num\r\n");
      vTaskList((char *)&buff);
      printf("%s\r\n",buff);
      printf("/*----------------------------------------------------------------------------*/\r\n");
      printf("Task        Runing_Num       Usage_Rate\r\n");
      vTaskGetRunTimeStats((char *)&buff);
      printf("%s\r\n",buff);
    }
    vTaskDelay(10);
  }
}

