/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.7
  * @date     2022-02-11
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32f403a_407_clock.h"
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "timer.h"
#include "usart.h"
#include "key.h"

/* ʼȼ*/
#define START_TASK_PRIO		1
/* ʼջС */
#define START_STK_SIZE 		128  
/* ʼ */
TaskHandle_t StartTask_Handler;
/* ʼں */
void start_task(void *pvParameters);

/* ͨȼ */
#define Ordinary_TASK_PRIO		3
/* ͨջС */
#define Ordinary_STK_SIZE 		256  
/* ͨ */
TaskHandle_t Ordinary_Task_Handler;
/* ͨں */
void Ordinary_task(void *pvParameters);

/* ݹȼ */
#define Recursive_TASK_PRIO		3
/* ݹջС */
#define Recursive_STK_SIZE 		256  
/* ݹ */
TaskHandle_t Recursive_Task_Handler;
/* ݹں */
void Recursive_task(void *pvParameters);

/* ȼ */
#define Debug_TASK_PRIO		4
/* ջС	*/
#define Debug_STK_SIZE 		512
/*  */
TaskHandle_t DebugTask_Handler;
/* ں */
void debug_task(void *pvParameters);

/* ź */
SemaphoreHandle_t AT_xSemaphore;


/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);
  
  system_clock_config();
  
  at32_led_init(LED2);
  at32_led_init(LED3);
  at32_led_init(LED4);
  
  /* ʼںͰ */
  USART_Init(115200);
  KEY_Init();
  
  /* ʼ */
  xTaskCreate((TaskFunction_t )start_task,
              (const char*    )"start_task",
              (uint16_t       )START_STK_SIZE,
              (void*          )NULL,
              (UBaseType_t    )START_TASK_PRIO,
              (TaskHandle_t*  )&StartTask_Handler);  
  /*  */              
  vTaskStartScheduler();
}
/* ʼ */
void start_task(void *pvParameters)
{
  /* ٽ */
  taskENTER_CRITICAL(); 
  
  /* ݹ黥ź */
  AT_xSemaphore = xSemaphoreCreateRecursiveMutex();
  
  if( AT_xSemaphore == NULL )
  {
    /* ݹ黥źʧ */
    while(1);
  }

  /* ڴݹ黥ź֮ٳʼʱ */
  TIMER_Init();
  
  /* ݹ */
  xTaskCreate((TaskFunction_t )Recursive_task,     
              (const char*    )"Recursive_task",   
              (uint16_t       )Recursive_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Recursive_TASK_PRIO,
              (TaskHandle_t*  )&Recursive_Task_Handler);   
  /* ͨ */
  xTaskCreate((TaskFunction_t )Ordinary_task,     
              (const char*    )"Ordinary_task",   
              (uint16_t       )Ordinary_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Ordinary_TASK_PRIO,
              (TaskHandle_t*  )&Ordinary_Task_Handler);               
  /*  */
  xTaskCreate((TaskFunction_t )debug_task,     
              (const char*    )"Debug_task",   
              (uint16_t       )Debug_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Debug_TASK_PRIO,
              (TaskHandle_t*  )&DebugTask_Handler);
  /* ɾʼ */            
  vTaskDelete(StartTask_Handler); 
  /* ˳ٽ */            
  taskEXIT_CRITICAL();            
}

/* ͨ */
void Ordinary_task(void *pvParameters)
{
 
  while(1)
  {
    vTaskDelay(5);
    
    if(xSemaphoreTakeRecursive( AT_xSemaphore, 5 ) == pdTRUE)
    {
      printf("Ordinary task received the recursive mutex semaphore.\r\n");
    }
    else
    {
      printf("Ordinary task didn't get the recursive mutex semaphore.\r\n");
    }
   
  }
}
/* ݹ */
void Recursive_task(void *pvParameters)
{
  while(1)
  {
    if( AT_xSemaphore != NULL )
    {
      if( xSemaphoreTakeRecursive( AT_xSemaphore, portMAX_DELAY ) == pdTRUE )
      {
        printf("Get the recursive mutex semaphore for the first time.\r\n");
        if(xSemaphoreTakeRecursive( AT_xSemaphore, portMAX_DELAY ) == pdTRUE )
        {
          printf("Get the recursive mutex semaphore for the second time.\r\n");
        }
        if(xSemaphoreTakeRecursive( AT_xSemaphore, portMAX_DELAY ) == pdTRUE )
        {
          printf("Get the recursive mutex semaphore for the third time.\r\n");
        }
      }
      vTaskDelay(5);
      xSemaphoreGiveRecursive( AT_xSemaphore );
      printf("Release the recursive mutex semaphore for the first time.\r\n");
      xSemaphoreGiveRecursive( AT_xSemaphore );
      printf("Release the recursive mutex semaphore for the second time.\r\n");
      xSemaphoreGiveRecursive( AT_xSemaphore );
      printf("Release the recursive mutex semaphore for the third time.\r\n");
    }
    vTaskDelay(100);
  }  
}
/*  */
void debug_task(void *pvParameters)
{
  u8 buff[500];
  while(1)
  {
    /* °ӡһεϢ */
    if(key_press() == USER_BUTTON)
    {
      printf("/*-----------------------------------------------------------------------------*/\r\n");
      printf("Task                 Status          priority      Remaining_Stack     Num\r\n");
      vTaskList((char *)&buff);
      printf("%s\r\n",buff);
      printf("/*----------------------------------------------------------------------------*/\r\n");
      printf("Task        Runing_Num       Usage_Rate\r\n");
      vTaskGetRunTimeStats((char *)&buff);
      printf("%s\r\n",buff);
    }
    vTaskDelay(10);
  }
}

