/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.7
  * @date     2022-02-11
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32f403a_407_clock.h"
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "event_groups.h"
#include "timer.h"
#include "usart.h"
#include "key.h"

/* ʼȼ*/
#define START_TASK_PRIO		1
/* ʼջС */
#define START_STK_SIZE 		128  
/* ʼ */
TaskHandle_t StartTask_Handler;
/* ʼں */
void start_task(void *pvParameters);

/* ¼־1ȼ */
#define EventGroup1_TASK_PRIO		3
/* ¼־1ջС */
#define EventGroup1_STK_SIZE 		256  
/* ¼־1 */
TaskHandle_t EventGroup1Task_Handler;
/* ¼־1ں */
void EventGroup1_task(void *pvParameters);

/* ¼־2ȼ */
#define EventGroup2_TASK_PRIO		3
/* ¼־2ջС */
#define EventGroup2_STK_SIZE 		256  
/* ¼־2 */
TaskHandle_t EventGroup2Task_Handler;
/* ¼־2ں */
void EventGroup2_task(void *pvParameters);

/* ¼־ȼ */
#define EventGroup_Receive_TASK_PRIO		4
/* ¼־ջС */
#define EventGroup_Receive_STK_SIZE 		256  
/* ¼־ */
TaskHandle_t EventGroup_ReceiveTask_Handler;
/* ¼־ں */
void EventGroup_Receive_task(void *pvParameters);

/* ȼ */
#define Debug_TASK_PRIO		5
/* ջС	*/
#define Debug_STK_SIZE 		512
/*  */
TaskHandle_t DebugTask_Handler;
/* ں */
void debug_task(void *pvParameters);

/* ͵BIT */
#define TASK_0_BIT ( 1 << 0 )
#define TASK_1_BIT ( 1 << 1 )
#define TASK_2_BIT ( 1 << 2 )

/* ͬ־ */
#define ALL_SYNC_BITS ( TASK_0_BIT | TASK_1_BIT | TASK_2_BIT )

/* һ¼־ */
EventGroupHandle_t AT_xEventBits;


/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);
  
  system_clock_config();
  
  at32_led_init(LED2);
  at32_led_init(LED3);
  at32_led_init(LED4);
  
  /* ʼںͰ */
  USART_Init(115200);
  KEY_Init();
  
  /* ʼ */
  xTaskCreate((TaskFunction_t )start_task,
              (const char*    )"start_task",
              (uint16_t       )START_STK_SIZE,
              (void*          )NULL,
              (UBaseType_t    )START_TASK_PRIO,
              (TaskHandle_t*  )&StartTask_Handler);  
  /*  */              
  vTaskStartScheduler();
}
/* ʼ */
void start_task(void *pvParameters)
{
  /* ٽ */
  taskENTER_CRITICAL(); 

  /* һ¼־ */
  AT_xEventBits = xEventGroupCreate();
  
  if(AT_xEventBits == NULL)
  {
    /* ʧ */
    while(1);
  }
  
  /* ڴϢ֮ٳʼʱ */
  TIMER_Init();
  /* ¼־1 */
  xTaskCreate((TaskFunction_t )EventGroup1_task,     
              (const char*    )"EventGroup1_task",   
              (uint16_t       )EventGroup1_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )EventGroup1_TASK_PRIO,
              (TaskHandle_t*  )&EventGroup1Task_Handler);  
  /* ¼־2 */
  xTaskCreate((TaskFunction_t )EventGroup2_task,     
              (const char*    )"EventGroup2_task",   
              (uint16_t       )EventGroup2_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )EventGroup2_TASK_PRIO,
              (TaskHandle_t*  )&EventGroup2Task_Handler);
  /* ¼־ */
  xTaskCreate((TaskFunction_t )EventGroup_Receive_task,     
              (const char*    )"EventReceive_task",   
              (uint16_t       )EventGroup_Receive_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )EventGroup_Receive_TASK_PRIO,
              (TaskHandle_t*  )&EventGroup_ReceiveTask_Handler);              
  /*  */
  xTaskCreate((TaskFunction_t )debug_task,     
              (const char*    )"Debug_task",   
              (uint16_t       )Debug_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Debug_TASK_PRIO,
              (TaskHandle_t*  )&DebugTask_Handler);
  /* ɾʼ */            
  vTaskDelete(StartTask_Handler); 
  /* ˳ٽ */            
  taskEXIT_CRITICAL();            
}
/* ¼־1 */
void EventGroup1_task(void *pvParameters)
{
  EventBits_t uxBits;
  
  while(1)
  {
    /* ʱ־BIT0&BIT2 */
    uxBits = xEventGroupSetBits( AT_xEventBits, TASK_0_BIT | TASK_2_BIT );    
    /* LED2 ON */
    at32_led_on(LED2);
    vTaskDelay(1000);    
    if( ( uxBits & ( TASK_0_BIT | TASK_2_BIT ) ) == ( TASK_0_BIT | TASK_2_BIT ) )
    {
      printf("Both bit 0 and bit 2 remained set when the function returned.\r\n");       
    }
    else if( ( uxBits & TASK_0_BIT ) != 0 )
    {
      printf("Bit 0 remained set when the function returned, but bit 2 was cleared.\r\n");
    }
    else if( ( uxBits & TASK_2_BIT ) != 0 )
    {
      printf("Bit 2 remained set when the function returned, but bit 0 was cleared.\r\n");
    }
    else
    {
      printf("Neither bit 0 nor bit 2 remained set.\r\n");
    }
    
  }
}
/* ¼־2 */
void EventGroup2_task(void *pvParameters)
{
  EventBits_t uxBits;
  
  while(1)
  {
    vTaskDelay(1500); 
    /* ʱ־BIT1 */
    uxBits = xEventGroupSetBits( AT_xEventBits, TASK_1_BIT );
    /* LED3 ON */
    at32_led_on(LED3);
    
    if( ( uxBits & TASK_1_BIT ) == TASK_1_BIT )
    {
      printf("Bit 1 remained set when the function returned.\r\n");
    }
    else
    {
      printf(" Bit 1 was cleared when the function returned.\r\n");
    }
  }
}

/* ¼־ */
void EventGroup_Receive_task(void *pvParameters)
{
  EventBits_t uxBits;
  
  while(1)
  {
    /* ALL_SYNC_BITS,ͬ */
    uxBits = xEventGroupWaitBits( AT_xEventBits, ALL_SYNC_BITS, pdTRUE, pdTRUE, portMAX_DELAY );
    /* LED4 ON */
    at32_led_on(LED4);
    printf("ALL_SYNC_BITS are received.\r\n");
  }
}
/*  */
void debug_task(void *pvParameters)
{
  u8 buff[500];
  while(1)
  {
    /* °ӡһεϢ */
    if(key_press() == USER_BUTTON)
    {
      printf("/*-----------------------------------------------------------------------------*/\r\n");
      printf("Task                 Status          priority      Remaining_Stack     Num\r\n");
      vTaskList((char *)&buff);
      printf("%s\r\n",buff);
      printf("/*----------------------------------------------------------------------------*/\r\n");
      printf("Task        Runing_Num       Usage_Rate\r\n");
      vTaskGetRunTimeStats((char *)&buff);
      printf("%s\r\n",buff);
    }
    vTaskDelay(10);
  }
}

