/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.7
  * @date     2022-02-11
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32f403a_407_clock.h"
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "event_groups.h"
#include "timer.h"
#include "usart.h"
#include "key.h"

/* ʼȼ*/
#define START_TASK_PRIO		1
/* ʼջС */
#define START_STK_SIZE 		128  
/* ʼ */
TaskHandle_t StartTask_Handler;
/* ʼں */
void start_task(void *pvParameters);

/* ʱȼ */
#define SoftwareTimer_Create_TASK_PRIO		4
/* ʱջС */
#define SoftwareTimer_Create_STK_SIZE 		256  
/* ʱ */
TaskHandle_t SoftwareTimer_CreateTask_Handler;
/* ʱں */
void SoftwareTimer_Create_task(void *pvParameters);

/* ȼ */
#define Debug_TASK_PRIO		5
/* ջС	*/
#define Debug_STK_SIZE 		512
/*  */
TaskHandle_t DebugTask_Handler;
/* ں */
void debug_task(void *pvParameters);

/* 贴ʱ */
#define NUM_TIMERS 5
/* ʱ */
TimerHandle_t AT_xTimers[ NUM_TIMERS ];
/* ʱ */
char *Timer_Name[] = {"Timer1","Timer2","Timer3","Timer4","Timer5"};
/* ʱص */
void AT_vTimerCallback( TimerHandle_t xTimer );

/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);
  
  system_clock_config();
  
  at32_led_init(LED2);
  at32_led_init(LED3);
  at32_led_init(LED4);
  
  /* ʼںͰ */
  USART_Init(115200);
  KEY_Init();
  
  /* ʼ */
  xTaskCreate((TaskFunction_t )start_task,
              (const char*    )"start_task",
              (uint16_t       )START_STK_SIZE,
              (void*          )NULL,
              (UBaseType_t    )START_TASK_PRIO,
              (TaskHandle_t*  )&StartTask_Handler);  
  /*  */              
  vTaskStartScheduler();
}

/* ʼ */
void start_task(void *pvParameters)
{
  /* ٽ */
  taskENTER_CRITICAL(); 
  
  /* ڴϢ֮ٳʼʱ */
  TIMER_Init();
  /* ʱ */
  xTaskCreate((TaskFunction_t )SoftwareTimer_Create_task,     
              (const char*    )"SoftwareTimer_task",   
              (uint16_t       )SoftwareTimer_Create_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )SoftwareTimer_Create_TASK_PRIO,
              (TaskHandle_t*  )&SoftwareTimer_CreateTask_Handler);              
  /*  */
  xTaskCreate((TaskFunction_t )debug_task,     
              (const char*    )"Debug_task",   
              (uint16_t       )Debug_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Debug_TASK_PRIO,
              (TaskHandle_t*  )&DebugTask_Handler);
  /* ɾʼ */            
  vTaskDelete(StartTask_Handler); 
  /* ˳ٽ */            
  taskEXIT_CRITICAL();            
}

/* ʱ */
void SoftwareTimer_Create_task(void *pvParameters)
{
  int i;
  /* ٽ */
  taskENTER_CRITICAL(); 
  for( i=0; i<NUM_TIMERS; i++ )
  {
    /* 5ʱ,ѭģʽ */
    AT_xTimers[ i ] = xTimerCreate( Timer_Name[i],
                                    ( 1000 * i ) + 1000,
                                    pdTRUE,
                                    ( void * ) 0,
                                    AT_vTimerCallback );
    if( AT_xTimers[ i ] == NULL )
    {
      /* ʧ */
      while(1);
    }
    else
    {
      if( xTimerStart( AT_xTimers[ i ], portMAX_DELAY ) != pdPASS )
      {
        /* ʱʧ */
        while(1);
      }
    }                                
  }
  
  /* ɾʱ */            
  vTaskDelete(SoftwareTimer_CreateTask_Handler);   
  /* ˳ٽ */            
  taskEXIT_CRITICAL();  
}
/*  */
void debug_task(void *pvParameters)
{
  u8 buff[500];
  while(1)
  {
    /* °ӡһεϢ */
    if(key_press() == USER_BUTTON)
    {
      printf("/*-----------------------------------------------------------------------------*/\r\n");
      printf("Task                 Status          priority      Remaining_Stack     Num\r\n");
      vTaskList((char *)&buff);
      printf("%s\r\n",buff);
      printf("/*----------------------------------------------------------------------------*/\r\n");
      printf("Task        Runing_Num       Usage_Rate\r\n");
      vTaskGetRunTimeStats((char *)&buff);
      printf("%s\r\n",buff);
    }
    vTaskDelay(10);
  }
}

/* ʱص */
void AT_vTimerCallback( TimerHandle_t xTimer )
{
  const uint32_t ulMaxExpiryCountBeforeStopping = 10;
  uint32_t ulCount;
  /* ȡʱID */
  ulCount = ( uint32_t ) pvTimerGetTimerID( xTimer );
  
  ulCount++;
  
  if( ulCount >= ulMaxExpiryCountBeforeStopping )
  {
    /* رʱ */
    xTimerStop( xTimer, 0 );
    /* ȡʱֲӡ */
    printf("The stop timer is %s\r\n",pcTimerGetName(xTimer));
    at32_led_toggle(LED4);
  }
  else
  {
    /* ʱID */
    vTimerSetTimerID( xTimer, ( void * ) ulCount );
    at32_led_toggle(LED2);
  }
}

