/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.7
  * @date     2022-02-11
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32f403a_407_clock.h"
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "timer.h"
#include "usart.h"
#include "key.h"

/* ʼȼ*/
#define START_TASK_PRIO		1
/* ʼջС */
#define START_STK_SIZE 		128  
/* ʼ */
TaskHandle_t StartTask_Handler;
/* ʼں */
void start_task(void *pvParameters);

/* Ϣȼ */
#define Process_Message_TASK_PRIO		1
/* ϢջС */
#define Process_Message_STK_SIZE 		256  
/* Ϣ */
TaskHandle_t Process_MessageTask_Handler;
/* Ϣں */
void Process_Message_task(void *pvParameters);

/* Ϣȼ */
#define Receive_Massage_TASK_PRIO		3
/* ϢջС */
#define Receive_Massage_STK_SIZE 		256  
/* Ϣ */
TaskHandle_t Receive_MassageTask_Handler;
/* Ϣں */
void Receive_Massage_task(void *pvParameters);


/* ȼ */
#define Debug_TASK_PRIO		3
/* ջС	*/
#define Debug_STK_SIZE 		512
/*  */
TaskHandle_t DebugTask_Handler;
/* ں */
void debug_task(void *pvParameters);

/* һϢṹ */
typedef struct A_Message
{
  char ucMessageID;
  u8 ucData;
} AMessage;

/* һϢ */
QueueHandle_t AT_xQueue;
/* ϢгȺϢС */
#define QUEUE_LENGTH 5
#define QUEUE_ITEM_SIZE sizeof( AMessage )
  
/* ͹ */
void Enter_Low_Power_Mode(uint32_t xExpectedIdleTime);
void Output_Low_Power_Mode(uint32_t xExpectedIdleTime);

void Enter_Low_Power_Mode(uint32_t xExpectedIdleTime)
{
  printf("Enter low power mode.\r\n");
  /* ûɸ,رʱ,ƵȲ */
}
void Output_Low_Power_Mode(uint32_t xExpectedIdleTime)
{
  printf("Output low power mode.\r\n");
  /* ûָ͹֮ǰ״̬ */
}

  
/* ڽܻС */
#define UART_RX_SIZE  512
#define NODATA   0xff

/* ڽܻ */
u8 UART_RX_BUF[UART_RX_SIZE];
u16 g_hwUartRxIndex;
u16 g_hwUartReadIndex;

/*------------------------------------------------------------------*/
void USART1_IRQHandler(void)
{
  if(usart_flag_get(USART1, USART_RDBF_FLAG) != RESET)
	{
    u8 Res = (u8)usart_data_receive(USART1);
    UART_RX_BUF[g_hwUartRxIndex] = Res;
    g_hwUartRxIndex = ( (g_hwUartRxIndex+1) & (UART_RX_SIZE-1) );
  }
}
/*------------------------------------------------------------------*/
u8 GetUartData(void)
{
	u8 hwData;
	
	if(g_hwUartReadIndex == g_hwUartRxIndex)
		hwData = NODATA;
	else
	{
		hwData = UART_RX_BUF[g_hwUartReadIndex];
		g_hwUartReadIndex = ( (g_hwUartReadIndex+1) & (UART_RX_SIZE-1) );	
	}
	return hwData;
}
/*------------------------------------------------------------------*/

/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);
  
  system_clock_config();
  
  at32_led_init(LED2);
  at32_led_init(LED3);
  at32_led_init(LED4);
  
  /* ʼںͰ */
  USART_Init(115200);
  KEY_Init();
  
  /* ʼ */
  xTaskCreate((TaskFunction_t )start_task,
              (const char*    )"start_task",
              (uint16_t       )START_STK_SIZE,
              (void*          )NULL,
              (UBaseType_t    )START_TASK_PRIO,
              (TaskHandle_t*  )&StartTask_Handler);  
  /*  */              
  vTaskStartScheduler();
}

/* ʼ */
void start_task(void *pvParameters)
{
  /* ٽ */
  taskENTER_CRITICAL(); 
  /* Ϣ */
  AT_xQueue = xQueueCreate( QUEUE_LENGTH, QUEUE_ITEM_SIZE );
  if(AT_xQueue == NULL)
  {
    /* Ϣдʧ */
    while(1);
  }
  /* ڴϢ֮ٳʼʱ */
  TIMER_Init();
  /* Ϣ */
  xTaskCreate((TaskFunction_t )Receive_Massage_task,     
              (const char*    )"Receive_Massage_task",   
              (uint16_t       )Receive_Massage_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Receive_Massage_TASK_PRIO,
              (TaskHandle_t*  )&Receive_MassageTask_Handler);  
  /* Ϣ */
  xTaskCreate((TaskFunction_t )Process_Message_task,     
              (const char*    )"Process_Message_task",   
              (uint16_t       )Process_Message_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Process_Message_TASK_PRIO,
              (TaskHandle_t*  )&Process_MessageTask_Handler);               
  /*  */
  xTaskCreate((TaskFunction_t )debug_task,     
              (const char*    )"Debug_task",   
              (uint16_t       )Debug_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Debug_TASK_PRIO,
              (TaskHandle_t*  )&DebugTask_Handler);
  /* ɾʼ */            
  vTaskDelete(StartTask_Handler); 
  /* ˳ٽ */            
  taskEXIT_CRITICAL();            
}
/* Ϣ */
void Receive_Massage_task(void *pvParameters)
{
  AMessage  Message1;
  while(1)
  {
      switch(GetUartData())
      {
        case NODATA:
          break;
        case 0x01:
          Message1.ucMessageID = 'a';
          Message1.ucData = 0x01;
          /* Toggle LED2Ϣ */
          xQueueSend(AT_xQueue,&Message1,10);
          break;
        case 0x02:
          Message1.ucMessageID = 'b';
          Message1.ucData = 0x02;
          /* Toggle LED3Ϣ */
          xQueueSend(AT_xQueue,&Message1,10);
          break;
        case 0x03:
          Message1.ucMessageID = 'c';
          Message1.ucData = 0x03;
          /* Toggle LED4Ϣ */
          xQueueSend(AT_xQueue,&Message1,10);
          break;
        default:
          break;
      }
    vTaskDelay(100);      
  }
}
/* Ϣ */
void Process_Message_task(void *pvParameters)
{
  AMessage  Message2;
  while(1)
  {
    /* Ϣ */
    if(xQueueReceive(AT_xQueue, &Message2, portMAX_DELAY) != pdPASS)
    {
      printf("no message\r\n");
    }
    else
    {
      /* жϢͲӦ */
      if((Message2.ucData == 0x01)&&(Message2.ucMessageID == 'a'))
        at32_led_toggle(LED2);
      if((Message2.ucData == 0x02)&&(Message2.ucMessageID == 'b'))
        at32_led_toggle(LED3);
      if((Message2.ucData == 0x03)&&(Message2.ucMessageID == 'c'))
        at32_led_toggle(LED4);
      if((Message2.ucData == 0x04)&&(Message2.ucMessageID == 'd'))
        printf("Timer interrupt\r\n");
    }
  }
}
/*  */
void debug_task(void *pvParameters)
{
  u8 buff[500];
  while(1)
  {
    /* °ӡһεϢ */
    if(key_press() == USER_BUTTON)
    {
      printf("/*-----------------------------------------------------------------------------*/\r\n");
      printf("Task                 Status          priority      Remaining_Stack     Num\r\n");
      vTaskList((char *)&buff);
      printf("%s\r\n",buff);
      printf("/*----------------------------------------------------------------------------*/\r\n");
      printf("Task        Runing_Num       Usage_Rate\r\n");
      vTaskGetRunTimeStats((char *)&buff);
      printf("%s\r\n",buff);
    }
    vTaskDelay(10);
  }
}

void TMR3_GLOBAL_IRQHandler(void)
{
  AMessage  Message3;
  BaseType_t xHigherPriorityTaskWoken = pdFALSE;
  if(tmr_flag_get(TMR3,TMR_OVF_FLAG)==SET)
  {
    Message3.ucMessageID = 'd';
    Message3.ucData = 0x04;
    /* ͶʱжϢϢ */
    xQueueSendFromISR(AT_xQueue,&Message3,&xHigherPriorityTaskWoken);
    /* жǷҪл */
    portYIELD_FROM_ISR(xHigherPriorityTaskWoken);
    tmr_flag_clear(TMR3,TMR_OVF_FLAG);
  }
}

