/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.7
  * @date     2022-02-11
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32f403a_407_clock.h"
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "event_groups.h"
#include "timer.h"
#include "usart.h"
#include "key.h"
#include "stream_buffer.h"

/* ʼȼ*/
#define START_TASK_PRIO		1
/* ʼջС */
#define START_STK_SIZE 		128  
/* ʼ */
TaskHandle_t StartTask_Handler;
/* ʼں */
void start_task(void *pvParameters);

/* Stream Buffersȼ */
#define Stream_Buffers_Send_TASK_PRIO		3
/* Stream BuffersջС */
#define Stream_Buffers_Send_STK_SIZE 		256  
/* Stream Buffers */
TaskHandle_t Stream_Buffers_SendTask_Handler;
/* Stream Buffersں */
void Stream_Buffers_Send_task(void *pvParameters);

/* Stream Buffersȼ */
#define Stream_Buffers_Receive_TASK_PRIO		3
/* Stream BuffersջС */
#define Stream_Buffers_Receive_STK_SIZE 		256  
/* Stream Buffers */
TaskHandle_t Stream_Buffers_ReceiveTask_Handler;
/* Stream Buffersں */
void Stream_Buffers_Receive_task(void *pvParameters);


/* ȼ */
#define Debug_TASK_PRIO		3
/* ջС	*/
#define Debug_STK_SIZE 		512
/*  */
TaskHandle_t DebugTask_Handler;
/* ں */
void debug_task(void *pvParameters);

/* һStream Buffer */
StreamBufferHandle_t AT_xStreamBuffer;
const size_t AT_xStreamBufferSizeBytes = 100, AT_xTriggerLevel = 1;


/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);
  
  system_clock_config();
  
  at32_led_init(LED2);
  at32_led_init(LED3);
  at32_led_init(LED4);
 
  /* ʼںͰ */
  USART_Init(115200);
  KEY_Init();
  
  /* ʼ */
  xTaskCreate((TaskFunction_t )start_task,
              (const char*    )"start_task",
              (uint16_t       )START_STK_SIZE,
              (void*          )NULL,
              (UBaseType_t    )START_TASK_PRIO,
              (TaskHandle_t*  )&StartTask_Handler);  
  /*  */              
  vTaskStartScheduler();
}

/* ʼ */
void start_task(void *pvParameters)
{
  /* ٽ */
  taskENTER_CRITICAL(); 
  
  /* Stream Buffer */
  AT_xStreamBuffer = xStreamBufferCreate( AT_xStreamBufferSizeBytes, AT_xTriggerLevel );
  if( AT_xStreamBuffer == NULL )
  {
    /* Stream Bufferʧ */
    while(1);
  }
  /* ʼʱ */
  TIMER_Init();
  /* Stream Buffers */
  xTaskCreate((TaskFunction_t )Stream_Buffers_Receive_task,     
              (const char*    )"Receive_Stream_Buffers_task",   
              (uint16_t       )Stream_Buffers_Receive_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Stream_Buffers_Receive_TASK_PRIO,
              (TaskHandle_t*  )&Stream_Buffers_ReceiveTask_Handler);  
  /* Stream Buffers */
  xTaskCreate((TaskFunction_t )Stream_Buffers_Send_task,     
              (const char*    )"Send_Stream_Buffers_task",   
              (uint16_t       )Stream_Buffers_Send_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Stream_Buffers_Send_TASK_PRIO,
              (TaskHandle_t*  )&Stream_Buffers_SendTask_Handler);               
  /*  */
  xTaskCreate((TaskFunction_t )debug_task,     
              (const char*    )"Debug_task",   
              (uint16_t       )Debug_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Debug_TASK_PRIO,
              (TaskHandle_t*  )&DebugTask_Handler);
  /* ɾʼ */            
  vTaskDelete(StartTask_Handler); 
  /* ˳ٽ */            
  taskEXIT_CRITICAL();            
}
/* Stream Buffers */
void Stream_Buffers_Receive_task(void *pvParameters)
{
  uint8_t ucRxData[ 2 ] = {'0','0'};
  size_t xReceivedBytes;
  const TickType_t xBlockTime = pdMS_TO_TICKS( 20 );
  while(1)
  {
    /* AT_xStreamBuffer */
    xReceivedBytes = xStreamBufferReceive( AT_xStreamBuffer, ( void * ) ucRxData, sizeof( ucRxData ), xBlockTime );
    if( xReceivedBytes > 0 )
    {
      printf("%s\r\n",ucRxData);
    }
  }
}
/* Stream Buffers */
void Stream_Buffers_Send_task(void *pvParameters)
{
  size_t xBytesSent;
  uint8_t ucArrayToSend[] = { 'A', 'T', '3', '2' };
  const TickType_t x100ms = pdMS_TO_TICKS( 100 );
  while(1)
  {
    /* ucArrayToSend[]е */
    xBytesSent = xStreamBufferSend( AT_xStreamBuffer, ( void * ) ucArrayToSend, sizeof( ucArrayToSend ), x100ms );   
    if( xBytesSent != sizeof( ucArrayToSend ) )
    {
      printf("#1:%d data written in.\r\n",xBytesSent);
    }
    vTaskDelay(3000);
  }
}
/*  */
void debug_task(void *pvParameters)
{
  u8 buff[500];
  while(1)
  {
    /* °ӡһεϢ */
    if(key_press() == USER_BUTTON)
    {
      printf("/*-----------------------------------------------------------------------------*/\r\n");
      printf("Task                 Status          priority      Remaining_Stack     Num\r\n");
      vTaskList((char *)&buff);
      printf("%s\r\n",buff);
      printf("/*----------------------------------------------------------------------------*/\r\n");
      printf("Task        Runing_Num       Usage_Rate\r\n");
      vTaskGetRunTimeStats((char *)&buff);
      printf("%s\r\n",buff);
    }
    vTaskDelay(10);
  }
}


