/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.7
  * @date     2022-02-11
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32f403a_407_clock.h"
#include "string.h"
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "event_groups.h"
#include "timer.h"
#include "usart.h"
#include "key.h"
#include "stream_buffer.h"
#include "message_buffer.h"

/* Task1ȼ*/
#define Task1_TASK_PRIO		2
/* Task1ջС */
#define Task1_STK_SIZE 		128  
/* Task1 */
TaskHandle_t AT_Task1_Handler;
/* Task1ں */
void AT_Task1(void *pvParameters);

/* Task2ȼ*/
#define Task2_TASK_PRIO		2
/* Task2ջС */
#define Task2_STK_SIZE 		128  
/* Task2 */
TaskHandle_t AT_Task2_Handler;
/* Task2ں */
void AT_Task2(void *pvParameters);

/* Task3ȼ*/
#define Task3_TASK_PRIO		2
/* Task3ջС */
#define Task3_STK_SIZE 		128  
/* Task3 */
TaskHandle_t AT_Task3_Handler;
/* Task3ں */
void AT_Task3(void *pvParameters);

/* ȼ */
#define Debug_TASK_PRIO		3
/* ջС	*/
#define Debug_STK_SIZE 		512
/*  */
TaskHandle_t DebugTask_Handler;
/* ں */
void debug_task(void *pvParameters);

#define TX_BIT 0x01
#define RX_BIT 0x02
static int Counter;
static uint32_t ulNotifiedValue;


/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);
  
  system_clock_config();
  
  at32_led_init(LED2);
  at32_led_init(LED3);
  at32_led_init(LED4);
   
  /* ʼںͰ */
  USART_Init(115200);
  KEY_Init();
  
   /* ٽ */
  taskENTER_CRITICAL(); 
  
  /* ʼʱ */
  TIMER_Init();
  
  /* Task1 */
  xTaskCreate((TaskFunction_t )AT_Task1,     
              (const char*    )"Task1",   
              (uint16_t       )Task1_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Task1_TASK_PRIO,
              (TaskHandle_t*  )&AT_Task1_Handler);
  /* Task2 */
  xTaskCreate((TaskFunction_t )AT_Task2,     
              (const char*    )"Task2",   
              (uint16_t       )Task2_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Task2_TASK_PRIO,
              (TaskHandle_t*  )&AT_Task2_Handler);
  /* Task3 */
  xTaskCreate((TaskFunction_t )AT_Task3,     
              (const char*    )"Task3",   
              (uint16_t       )Task3_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Task3_TASK_PRIO,
              (TaskHandle_t*  )&AT_Task3_Handler);
  /*  */
  xTaskCreate((TaskFunction_t )debug_task,     
              (const char*    )"Debug_task",   
              (uint16_t       )Debug_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Debug_TASK_PRIO,
              (TaskHandle_t*  )&DebugTask_Handler);   
  /*  */              
  vTaskStartScheduler();
}

/* Task1 */
void AT_Task1(void *pvParameters)
{
  while(1)
  {
    vTaskDelay(50);
    /* AT_Task2_Handler֪ͨ */
    printf("Send a notification to AT_Task2.\r\n");
    xTaskNotifyGive( AT_Task2_Handler );
    at32_led_toggle(LED4);
    vTaskDelay(1000);
    /* ȴAT_Task2_Handler֪ͨ */  
    ulTaskNotifyTake( pdTRUE, portMAX_DELAY );
    printf("Received the AT_Task2's task notification.\r\n"); 
  } 
}
/* Task2 */
void AT_Task2(void *pvParameters)
{
  while(1)
  {
    vTaskDelay(100);
    /* ȴAT_Task1_Handler֪ͨ */   
    ulTaskNotifyTake( pdTRUE, portMAX_DELAY );
    printf("Received the AT_Task1's task notification.\r\n"); 
    vTaskDelay(2000);
    /* AT_Task1_Handler֪ͨ */
    printf("Send a notification to AT_Task1.\r\n");    
    xTaskNotifyGive( AT_Task1_Handler );
    at32_led_toggle(LED4);
  }
}
/* Task3 */
void AT_Task3(void *pvParameters)
{
  BaseType_t xResult;
  while(1)
  {
    /* ȴ֪ͨ */
    xResult = xTaskNotifyWait( pdFALSE, 0xffffffff, &ulNotifiedValue, portMAX_DELAY );
    if( xResult == pdPASS )
    {
      /* ֪ܵͨ */
      if( ( ulNotifiedValue & TX_BIT ) != 0 )
      {
        at32_led_toggle(LED2);
        printf("The TX ISR has set a bit.\r\n");
      }
      if( ( ulNotifiedValue & RX_BIT ) != 0 )
      {
        at32_led_toggle(LED3);
        printf("The RX ISR has set a bit.\r\n");
      }
    }
  }
}
/* Ӳʱ3жϺ */
void TMR3_GLOBAL_IRQHandler(void)
{
  BaseType_t xHigherPriorityTaskWoken = pdFALSE;
  if(tmr_flag_get(TMR3,TMR_OVF_FLAG)==SET)
  {
    Counter++;
    if( (Counter % 2) == 0 )
    {
      printf("Notify the task by setting the RX_BIT in the task's notification value.\r\n");
      xTaskNotifyFromISR( AT_Task3_Handler, RX_BIT, eSetBits, &xHigherPriorityTaskWoken );
    }
    else
    {
      printf("Notify the task by setting the TX_BIT in the task's notification value.\r\n");
      xTaskNotifyFromISR( AT_Task3_Handler, TX_BIT, eSetBits, &xHigherPriorityTaskWoken );
    }

    tmr_flag_clear(TMR3,TMR_OVF_FLAG);
    /* жǷҪл */
    portYIELD_FROM_ISR( xHigherPriorityTaskWoken );
  }
}
/*  */
void debug_task(void *pvParameters)
{
  u8 buff[500];
  while(1)
  {
    /* °ӡһεϢ */
    if(key_press() == USER_BUTTON)
    {
      printf("/*-----------------------------------------------------------------------------*/\r\n");
      printf("Task                 Status          priority      Remaining_Stack     Num\r\n");
      vTaskList((char *)&buff);
      printf("%s\r\n",buff);
      printf("/*----------------------------------------------------------------------------*/\r\n");
      printf("Task        Runing_Num       Usage_Rate\r\n");
      vTaskGetRunTimeStats((char *)&buff);
      printf("%s\r\n",buff);
    }
    vTaskDelay(10);
  }
}


