/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.7
  * @date     2022-02-11
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32f403a_407_clock.h"
#include "string.h"
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "event_groups.h"
#include "timer.h"
#include "usart.h"
#include "key.h"
#include "stream_buffer.h"
#include "message_buffer.h"
#include "semphr.h"
#include "usbd_core.h"
#include "keyboard_class.h"
#include "keyboard_desc.h"
#include "usbd_int.h"

#define ADC_VREF                         (3.3)
#define ADC_TEMP_BASE                    (1.26)
#define ADC_TEMP_SLOPE                   (-0.00423)

__IO uint16_t adc1_ordinary_value = 0;

static void dma_config(void);
static void adc_config(void);

uint8_t Keyboad_Buf[8]={0,0,6,0,0,0,0,0};
u16 recvLen, sendLen;

usbd_core_type usb_core_dev;
void keyboard_send_string(uint8_t *string, uint8_t len);

/* ڽܻС */
#define UART_RX_SIZE  512
#define NODATA   0xff

/* ڽܻ */
u8 UART_RX_BUF[UART_RX_SIZE];
u16 g_hwUartRxIndex;
u16 g_hwUartReadIndex;

/* Keyboardȼ */
#define Keyboard_Task_PRIO		4
/* KeyboardջС	*/
#define Keyboard_STK_SIZE 		256  
/* Keyboard */ 
TaskHandle_t KeyboardTask_Handler;
/*Keyboard */
void Keyboard_task(void *pvParameters);

uint8_t result_1[] = "Hello World !";

/* Keyȼ */
#define Key_Task_PRIO		3
/* KeyջС	*/
#define Key_STK_SIZE 		256  
/* Key */ 
TaskHandle_t KeyTask_Handler;
/*Key */
void Key_task(void *pvParameters);

/* Ϣȼ */
#define Process_Message_TASK_PRIO		1
/* ϢջС */
#define Process_Message_STK_SIZE 		256  
/* Ϣ */
TaskHandle_t Process_MessageTask_Handler;
/* Ϣں */
void Process_Message_task(void *pvParameters);

/* Ϣȼ */
#define Receive_Massage_TASK_PRIO		3
/* ϢջС */
#define Receive_Massage_STK_SIZE 		256  
/* Ϣ */
TaskHandle_t Receive_MassageTask_Handler;
/* Ϣں */
void Receive_Massage_task(void *pvParameters);

/* һϢṹ */
typedef struct A_Message
{
  char ucMessageID;
  u8 ucData;
} AMessage;

/* һϢ */
QueueHandle_t AT_xQueue;
/* ϢгȺϢС */
#define QUEUE_LENGTH 5
#define QUEUE_ITEM_SIZE sizeof( AMessage )
  
/* һֵź */
SemaphoreHandle_t AT_xSemaphore;

/* һ,һζֵźһ */
int Take_Semaphore_Counter = 0;

/* һʱ */
TimerHandle_t AT_xTimers;
/* ʱص */
void AT_vTimerCallback( TimerHandle_t xTimer );



/*------------------------------------------------------------------*/
void USART1_IRQHandler(void)
{
  if(usart_flag_get(USART1, USART_RDBF_FLAG) != RESET)
	{
    u8 Res = (u8)usart_data_receive(USART1);
    UART_RX_BUF[g_hwUartRxIndex] = Res;
    g_hwUartRxIndex = ( (g_hwUartRxIndex+1) & (UART_RX_SIZE-1) );
  }
}
/*------------------------------------------------------------------*/
u8 GetUartData(void)
{
	u8 hwData;
	
	if(g_hwUartReadIndex == g_hwUartRxIndex)
		hwData = NODATA;
	else
	{
		hwData = UART_RX_BUF[g_hwUartReadIndex];
		g_hwUartReadIndex = ( (g_hwUartReadIndex+1) & (UART_RX_SIZE-1) );	
	}
	return hwData;
}
/*------------------------------------------------------------------*/


/**
  * @brief  keyboard send string
  * @param  string: send string
  * @param  len: send string length
  * @retval none
  */
void keyboard_send_string(uint8_t *string, uint8_t len)
{
  uint8_t index = 0;
  for(index = 0; index < len; index ++)
  {
    while(1)
    {
      if(g_u8tx_completed == 1)
      {
        g_u8tx_completed = 0;
        usb_hid_keyboard_send_char(&usb_core_dev, string[index]);
        break;
      }
    }
  }
  while(1)
  {
    if(g_u8tx_completed == 1)
    {
      g_u8tx_completed = 0;
      usb_hid_keyboard_send_char(&usb_core_dev, 0x00);
      break;
    }
  }
}

/**
  * @brief  usb 48M clock select
  * @param  clk_s:USB_CLK_HICK, USB_CLK_HEXT
  * @retval none
  */
void usb_clock48m_select(usb_clk48_s clk_s)
{
  if(clk_s == USB_CLK_HICK)
  {
    crm_usb_clock_source_select(CRM_USB_CLOCK_SOURCE_HICK);

    /* enable the acc calibration ready interrupt */
    crm_periph_clock_enable(CRM_ACC_PERIPH_CLOCK, TRUE);

    /* update the c1\c2\c3 value */
    acc_write_c1(7980);
    acc_write_c2(8000);
    acc_write_c3(8020);

    /* open acc calibration */
    acc_calibration_mode_enable(ACC_CAL_HICKTRIM, TRUE);
  }
  else
  {
    switch(system_core_clock)
    {
      /* 48MHz */
      case 48000000:
        crm_usb_clock_div_set(CRM_USB_DIV_1);
        break;

      /* 72MHz */
      case 72000000:
        crm_usb_clock_div_set(CRM_USB_DIV_1_5);
        break;

      /* 96MHz */
      case 96000000:
        crm_usb_clock_div_set(CRM_USB_DIV_2);
        break;

      /* 120MHz */
      case 120000000:
        crm_usb_clock_div_set(CRM_USB_DIV_2_5);
        break;

      /* 144MHz */
      case 144000000:
        crm_usb_clock_div_set(CRM_USB_DIV_3);
        break;

      /* 168MHz */
      case 168000000:
        crm_usb_clock_div_set(CRM_USB_DIV_3_5);
        break;

      /* 192MHz */
      case 192000000:
        crm_usb_clock_div_set(CRM_USB_DIV_4);
        break;

      default:
        break;

    }
  }
}


/**
  * @brief  dma configuration.
  * @param  none
  * @retval none
  */
static void dma_config(void)
{
  dma_init_type dma_init_struct;
  crm_periph_clock_enable(CRM_DMA1_PERIPH_CLOCK, TRUE);
  dma_reset(DMA1_CHANNEL1);
  dma_default_para_init(&dma_init_struct);
  dma_init_struct.buffer_size = 1;
  dma_init_struct.direction = DMA_DIR_PERIPHERAL_TO_MEMORY;
  dma_init_struct.memory_base_addr = (uint32_t)&adc1_ordinary_value;
  dma_init_struct.memory_data_width = DMA_MEMORY_DATA_WIDTH_HALFWORD;
  dma_init_struct.memory_inc_enable = FALSE;
  dma_init_struct.peripheral_base_addr = (uint32_t)&(ADC1->odt);
  dma_init_struct.peripheral_data_width = DMA_PERIPHERAL_DATA_WIDTH_HALFWORD;
  dma_init_struct.peripheral_inc_enable = FALSE;
  dma_init_struct.priority = DMA_PRIORITY_HIGH;
  dma_init_struct.loop_mode_enable = TRUE;
  dma_init(DMA1_CHANNEL1, &dma_init_struct);

  dma_channel_enable(DMA1_CHANNEL1, TRUE);
}

/**
  * @brief  adc configuration.
  * @param  none
  * @retval none
  */
static void adc_config(void)
{
  adc_base_config_type adc_base_struct;
  crm_periph_clock_enable(CRM_ADC1_PERIPH_CLOCK, TRUE);
  crm_adc_clock_div_set(CRM_ADC_DIV_6);

  /* select combine mode */
  adc_combine_mode_select(ADC_INDEPENDENT_MODE);
  adc_base_default_para_init(&adc_base_struct);
  adc_base_struct.sequence_mode = FALSE;
  adc_base_struct.repeat_mode = TRUE;
  adc_base_struct.data_align = ADC_RIGHT_ALIGNMENT;
  adc_base_struct.ordinary_channel_length = 1;
  adc_base_config(ADC1, &adc_base_struct);
  adc_ordinary_channel_set(ADC1, ADC_CHANNEL_16, 1, ADC_SAMPLETIME_239_5);
  adc_ordinary_conversion_trigger_set(ADC1, ADC12_ORDINARY_TRIG_SOFTWARE, TRUE);
  adc_dma_mode_enable(ADC1, TRUE);
  adc_tempersensor_vintrv_enable(TRUE);

  adc_enable(ADC1, TRUE);
  adc_calibration_init(ADC1);
  while(adc_calibration_init_status_get(ADC1));
  adc_calibration_start(ADC1);
  while(adc_calibration_status_get(ADC1));
}
/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);
  
  system_clock_config();
  
  at32_led_init(LED2);
  at32_led_init(LED3);
  at32_led_init(LED4);
  
  /* ʼںͰ */
  USART_Init(115200);
  KEY_Init();
  
  /* ٽ */
  taskENTER_CRITICAL(); 
  
  /* Ϣ */
  AT_xQueue = xQueueCreate( QUEUE_LENGTH, QUEUE_ITEM_SIZE );
  if(AT_xQueue == NULL)
  {
    /* Ϣдʧ */
    while(1);
  }
  
  /* ֵź */
  AT_xSemaphore = xSemaphoreCreateBinary(); 
  if( AT_xSemaphore == NULL )
  {
    /* ֵźʧ */
    while(1);
  }
  
  /* ʱ */
  AT_xTimers = xTimerCreate( "timer", 2000, pdTRUE, ( void * ) 0, AT_vTimerCallback );
  if(AT_xTimers == NULL)
  {
    /* ʱʧ */
    while(1);
  }
  else
  {
    if( xTimerStart( AT_xTimers, portMAX_DELAY ) != pdPASS )
      {
        /* ʱʧ */
        while(1);
      }
  }
  
  /* ʼʱ */
  TIMER_Init();
  
  /* ʼADC */
  dma_config();
  adc_config();
  
  /* Ϣ */
  xTaskCreate((TaskFunction_t )Receive_Massage_task,     
              (const char*    )"Receive_Massage_task",   
              (uint16_t       )Receive_Massage_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Receive_Massage_TASK_PRIO,
              (TaskHandle_t*  )&Receive_MassageTask_Handler);  
  /* Ϣ */
  xTaskCreate((TaskFunction_t )Process_Message_task,     
              (const char*    )"Process_Message_task",   
              (uint16_t       )Process_Message_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Process_Message_TASK_PRIO,
              (TaskHandle_t*  )&Process_MessageTask_Handler);
  /* Keyboard */
  xTaskCreate((TaskFunction_t )Keyboard_task,     
              (const char*    )"Keyboard_task",   
              (uint16_t       )Keyboard_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Keyboard_Task_PRIO,
              (TaskHandle_t*  )&KeyboardTask_Handler); 
  /* Key */
  xTaskCreate((TaskFunction_t )Key_task,     
              (const char*    )"Key_task",   
              (uint16_t       )Key_STK_SIZE, 
              (void*          )NULL,
              (UBaseType_t    )Key_Task_PRIO,
              (TaskHandle_t*  )&KeyTask_Handler);          
   
  /* ˳ٽ */            
  taskEXIT_CRITICAL();  
  /*  */              
  vTaskStartScheduler();
}

/* Keyboard */
void Keyboard_task(void *pvParameters)
{
  /* USBسʼ */
  /* select usb 48m clcok source */
  usb_clock48m_select(USB_CLK_HEXT);
  
  /* enable usb clock */
  crm_periph_clock_enable(CRM_USB_PERIPH_CLOCK, TRUE);

  /* enable usb interrupt */
  nvic_irq_enable(USBFS_L_CAN1_RX0_IRQn, 0, 0);

  /* usb core init */
  usbd_core_init(&usb_core_dev, USB, &keyboard_class_handler, &keyboard_desc_handler, 0);

  /* enable usb pull-up */
  usbd_connect(&usb_core_dev);
  
  while(1)
	{	
    /* ȴնֵź */
    if( xSemaphoreTake( AT_xSemaphore, portMAX_DELAY ) == pdTRUE )
    {
      /* յֵź,USB */
      if(usbd_connect_state_get(&usb_core_dev) == USB_CONN_STATE_CONFIGURED)
      {
        recvLen = strlen((const char*)result_1);
        keyboard_send_string(result_1, recvLen);
        /* ӡնֵź */
        Take_Semaphore_Counter++;
        printf("The %dth semaphore is received.\r\n",Take_Semaphore_Counter);
      }
    }
	} 
}

/* Keyboard */
void Key_task(void *pvParameters)
{
  while(1)
	{	
		/*  */
    if(key_press() == USER_BUTTON)
    {
      /* Ͷֵź */
      xSemaphoreGive( AT_xSemaphore);
    }
    vTaskDelay(10);
	} 
}

/* Ϣ */
void Receive_Massage_task(void *pvParameters)
{
  AMessage  Message1;
  while(1)
  {
    switch(GetUartData())
    {
      case NODATA:
        break;
      case 0x01:
        Message1.ucMessageID = 'a';
        Message1.ucData = 0x01;
        /* Toggle LED2Ϣ */
        xQueueSend(AT_xQueue,&Message1,10);
        break;
      case 0x02:
        Message1.ucMessageID = 'b';
        Message1.ucData = 0x02;
        /* Toggle LED3Ϣ */
        xQueueSend(AT_xQueue,&Message1,10);
        break;
      case 0x03:
        Message1.ucMessageID = 'c';
        Message1.ucData = 0x03;
        /* Toggle LED4Ϣ */
        xQueueSend(AT_xQueue,&Message1,10);
        break;
      default:
        break;
    }
    vTaskDelay(100);      
  }
}
/* Ϣ */
void Process_Message_task(void *pvParameters)
{
  AMessage  Message2;
  while(1)
  {
    /* Ϣ */
    if(xQueueReceive(AT_xQueue, &Message2, portMAX_DELAY) != pdPASS)
    {
      printf("no message\r\n");
    }
    else
    {
      /* жϢͲӦ */
      if((Message2.ucData == 0x01)&&(Message2.ucMessageID == 'a'))
        at32_led_toggle(LED2);
      if((Message2.ucData == 0x02)&&(Message2.ucMessageID == 'b'))
        at32_led_toggle(LED3);
      if((Message2.ucData == 0x03)&&(Message2.ucMessageID == 'c'))
        at32_led_toggle(LED4);
      if((Message2.ucData == 0x04)&&(Message2.ucMessageID == 'd'))
        printf("Timer interrupt\r\n");
    }
  }
}

/* ʱص,ɼһоƬڲ¶ȴֵӡ */
void AT_vTimerCallback( TimerHandle_t xTimer )
{
  printf("Software timer is update.\r\n");
  /* ADCתһ */ 
  adc_ordinary_software_trigger_enable(ADC1, TRUE);
  vTaskDelay(100);
  printf("internal_temperature = %f deg C\r\n",(ADC_TEMP_BASE - (double)adc1_ordinary_value * ADC_VREF / 4096) / ADC_TEMP_SLOPE + 25);
}

/* ʱжϴ */
void TMR3_GLOBAL_IRQHandler(void)
{
  AMessage  Message3;
  BaseType_t xHigherPriorityTaskWoken = pdFALSE;
  if(tmr_flag_get(TMR3,TMR_OVF_FLAG)==SET)
  {
    Message3.ucMessageID = 'd';
    Message3.ucData = 0x04;
    /* ͶʱжϢϢ */
    xQueueSendFromISR(AT_xQueue,&Message3,&xHigherPriorityTaskWoken);
    /* жǷҪл */
    portYIELD_FROM_ISR(xHigherPriorityTaskWoken);
    tmr_flag_clear(TMR3,TMR_OVF_FLAG);
  }
}


/**
  * @brief  this function handles usb interrupt.
  * @param  none
  * @retval none
  */
void USBFS_L_CAN1_RX0_IRQHandler(void)
{
  usbd_irq_handler(&usb_core_dev);
}
